"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          15-05-2018
//      Created for Project :   FLAME
"""

import pytest
from pyramid import testing
from pyramid.httpexceptions import HTTPBadRequest


class TestAggregatorConfig(object):
    """
    A pytest-implementation test for the aggregator configuration API calls
    """

    @pytest.fixture(autouse=True)
    def app_config(self):
        """
        A fixture to implement setUp/tearDown functionality for all tests by initializing configuration structure for the web service
        """

        self.config = testing.setUp()
        self.config.add_settings({'aggregator_running': False})

        yield

        testing.tearDown()

    def test_GET(self):
        """
        Tests the GET method for the status of the aggregator.
        """

        from clmcservice.views import AggregatorConfig  # nested import so that importing the class view is part of the test itself

        assert not self.config.get_settings().get('aggregator_running'), "Initially aggregator is not running."

        request = testing.DummyRequest()
        response = AggregatorConfig(request).get()

        assert type(response) == dict, "Response must be a dictionary representing a JSON object."
        assert not response.get('aggregator_running'), "The response of the API call must return the aggregator status being set as False"
        assert not self.config.get_settings().get('aggregator_running'), "A GET request must not modify the aggregator status."

    @pytest.mark.parametrize("input_val, output_val", [
        ("True", True),
        ("true", True),
        ("1", True),
        ("False", False),
        ("false", False),
        ("0", False),
        ("t", None),
        ("f", None),
    ])
    def test_POST(self, input_val, output_val):
        """
        Tests the POST method for the status of the aggregator
        :param input_val: the input form parameter
        :param output_val: the expected output value, None for expecting an Exception
        """

        from clmcservice.views import AggregatorConfig  # nested import so that importing the class view is part of the test itself

        assert not self.config.get_settings().get('aggregator_running'), "Initially aggregator is not running."

        request = testing.DummyRequest()

        request.params['running'] = input_val
        if output_val is not None:
            response = AggregatorConfig(request).post()
            assert response == {'aggregator_running': output_val}, "Response of POST request must include the new status of the aggregator"
            assert self.config.get_settings().get('aggregator_running') == output_val, "Aggregator status must be updated to running."
        else:
            error_raised = False
            try:
                AggregatorConfig(request).post()
            except HTTPBadRequest:
                error_raised = True

            assert error_raised, "Error must be raised in case of an invalid argument."
